package hr.seminar;

import hr.genetics.models.SimulationState;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.nio.file.Paths;
import java.util.AbstractMap;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class SimulationWriter {

	private String filenamePrefix = "";
	private String directoryName = null;
	private Integer pathID = 0;
	private String stopReason;
	
	private List<Map.Entry<Double, Double>> stats;
	
	public SimulationWriter() {
		
		java.util.Date date= new java.util.Date();

		this.directoryName = "/Users/mario/workspace/Seminar/runs/run_"+ new Double(date.getTime()/1000).intValue()+"/";
		
		try {
			new File(this.directoryName).mkdir();
		} catch (Exception e) {
			System.out.println(e);
		}
		
		this.stats = new ArrayList<>();
			
	}
	
	public void setPathID(Integer pathID) {
		this.pathID = pathID;
	}
	
	public void writeSimilation(List<SimulationState> states) {
		
		String filename = this.directoryName + this.filenamePrefix + "best" + ".txt";
		
		try (BufferedWriter fStream = new BufferedWriter(new FileWriter(Paths.get(filename).toFile()))) {
			
			fStream.write("PATH " + this.pathID + "\n");
			for (SimulationState state : states) {
				fStream.write(state.getPosition().getX() 
						+ " " + state.getPosition().getY()
						+ " " + state.getSpeed().getAngle() + "\n");
			}
			fStream.write(stopReason);
			
		} catch (Exception e) {
			e.printStackTrace();
		}
			
				
	}
	
	public SimulationWriter setFilenamePrefix(String prefix) {
		
		this.filenamePrefix = prefix;
		
		return this;
		
	}
	
	public SimulationWriter addStats(double avg, double max) {
		
		this.stats.add(
				new AbstractMap.SimpleEntry<Double, Double>(new Double(avg), new Double(max))
		);
		
		return this;
	}
	
	public void writeStatistics() {
		
		String filename = this.directoryName + "statistics.csv";
 
		try (BufferedWriter fStream = new BufferedWriter(new FileWriter(Paths.get(filename).toFile()))) {
			
			int i = 1;
			for (Map.Entry<Double, Double> pair : this.stats) {
				fStream.write(i + "," + pair.getKey() + "," + pair.getValue() + "\n");
				++i;
			}
			
		} catch (Exception e) {
			
		}
		
	}
	
	public void setStopReason(String stopReason) {
		this.stopReason = stopReason;
	}
	
}
